@@ -1,8 +1,6 @@
 package gnu.crypto.prng;
 
 // ----------------------------------------------------------------------------
-// $Id: BasePRNG.java,v 1.7 2002/11/07 17:17:45 raif Exp $
-//
 // Copyright (C) 2001, 2002, Free Software Foundation, Inc.
 //
 // This file is part of GNU Crypto.
@@ -21,8 +19,8 @@ package gnu.crypto.prng;
 // along with this program; see the file COPYING.  If not, write to the
 //
 //    Free Software Foundation Inc.,
-//    59 Temple Place - Suite 330,
-//    Boston, MA 02111-1307
+//    51 Franklin Street, Fifth Floor,
+//    Boston, MA 02110-1301
 //    USA
 //
 // Linking this library statically or dynamically with other modules is
@@ -48,7 +46,7 @@ import java.util.Map;
 /**
  * <p>An abstract class to facilitate implementing PRNG algorithms.</p>
  *
- * @version $Revision: 1.7 $
+ * @version $Revision: 1.12 $
  */
 public abstract class BasePRNG implements IRandom {
 
@@ -109,24 +107,50 @@ public abstract class BasePRNG implements IRandom {
       return nextByteInternal();
    }
 
+   public void nextBytes(byte[] out) throws IllegalStateException, LimitReachedException {
+      nextBytes(out, 0, out.length);
+   }
+
    public void nextBytes(byte[] out, int offset, int length)
-   throws IllegalStateException, LimitReachedException {
-      if (out == null) {
+   throws IllegalStateException, LimitReachedException
+   {
+      if (!initialised)
+         throw new IllegalStateException("not initialized");
+
+      if (length == 0)
          return;
-      }
 
-      if (!initialised) {
-         throw new IllegalStateException();
-      }
+      if (offset < 0 || length < 0 || offset + length > out.length)
+         throw new ArrayIndexOutOfBoundsException("offset=" + offset + " length="
+                                                  + length + " limit=" + out.length);
 
-      if (offset < 0 || offset >= out.length || length < 1) {
-         return;
+      if (ndx >= buffer.length) {
+         fillBlock();
+         ndx = 0;
       }
+      int count = 0;
+      while (count < length) {
+         int amount = Math.min(buffer.length - ndx, length - count);
+         System.arraycopy(buffer, ndx, out, offset+count, amount);
+         count += amount;
+         ndx += amount;
+         if (ndx >= buffer.length) {
+            fillBlock();
+            ndx = 0;
+         }
+      }
+   }
 
-      int limit = ((offset+length) > out.length ? out.length-offset : length);
-      for (int i = 0; i < limit; i++) {
-         out[offset++] = nextByteInternal();
-      }
+   public void addRandomByte(byte b) {
+      throw new UnsupportedOperationException("random state is non-modifiable");
+   }
+
+   public void addRandomBytes(byte[] buffer) {
+      addRandomBytes(buffer, 0, buffer.length);
+   }
+
+   public void addRandomBytes(byte[] buffer, int offset, int length) {
+      throw new UnsupportedOperationException("random state is non-modifiable");
    }
 
    // Instance methods
@@ -147,7 +171,10 @@ public abstract class BasePRNG implements IRandom {
 
    // abstract methods to implement by subclasses -----------------------------
 
-   public abstract Object clone();
+  public Object clone() throws CloneNotSupportedException
+  {
+    return super.clone();
+  }
 
    public abstract void setup(Map attributes);
 
